///////////////////////////////////////////////////////////////////////////
//	ONTOP.CPP		How to stay on top
///////////////////////////////////////////////////////////////////////////
#include <owl\applicat.h>
#include <owl\framewin.h>
#pragma hdrstop
#include "ontop.rh"

#define WINCLASSNAME "TAlwaysOnTop"

///////////////////////////////////////////////////////////////////////////
class TAlwaysOnTop: public TFrameWindow {
public:
	TAlwaysOnTop(TWindow* parent, const char far* title);
protected:
	void SetupWindow();
	virtual char far* GetClassName() {return WINCLASSNAME;}
	void SetOnTop();
	void EvActivate(UINT active, BOOL minimized, HWND hWndOther);
	void CmOnTop();
	void CmEnableOnTop(TCommandEnabler &ce);
private:
	BOOL onTop;
DECLARE_RESPONSE_TABLE(TAlwaysOnTop);
};

DEFINE_RESPONSE_TABLE1(TAlwaysOnTop, TFrameWindow)
	EV_WM_ACTIVATE,
	EV_COMMAND(CM_ONTOP, CmOnTop),
	EV_COMMAND_ENABLE(CM_ONTOP, CmEnableOnTop),
END_RESPONSE_TABLE;


TAlwaysOnTop::TAlwaysOnTop(TWindow* parent, const char far* title)
	: TFrameWindow(parent, title),
		TWindow(parent, title),
		onTop(TRUE)
{
	AssignMenu(ID_MENU);

	Attr.X = 50;
	Attr.Y = 50;
	Attr.W = 200;
	Attr.H = 150;
}


void TAlwaysOnTop::SetupWindow()
{
	TFrameWindow::SetupWindow();
	SetOnTop();
}


void TAlwaysOnTop::SetOnTop()
{
	HWND hwndInsertAfter;
	if (onTop)
		hwndInsertAfter = HWND_TOPMOST;
	else
		hwndInsertAfter = HWND_NOTOPMOST;
	SetWindowPos(hwndInsertAfter, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
}


void TAlwaysOnTop::EvActivate(UINT /*active*/,
	BOOL /*minimized*/, HWND /*hWndOther*/)
{
	if (onTop)
		::BringWindowToTop(GetLastActivePopup());
	DefaultProcessing();
}


void TAlwaysOnTop::CmOnTop()
{
	onTop = !onTop;
	if (onTop)
		SetCaption("Always on Top");
	else
		SetCaption("Not on Top");

	SetOnTop();
}


void TAlwaysOnTop::CmEnableOnTop(TCommandEnabler &ce)
{
	ce.SetCheck(onTop ?
		TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

///////////////////////////////////////////////////////////////////////////
class TOptionApp: public TApplication {
public:
	TOptionApp(const char far* name)
		: TApplication(name) {}
	void InitMainWindow();
	void InitInstance();
};


void TOptionApp::InitMainWindow()
{
	TAlwaysOnTop* optionWin = new TAlwaysOnTop(0, "Always on Top");
	MainWindow = optionWin;
}


void TOptionApp::InitInstance()
{
	if (hPrevInstance) {
		HWND hwnd = ::FindWindow(WINCLASSNAME, 0);
		if (hwnd) {
			hwnd = GetLastActivePopup(hwnd);
			BringWindowToTop(hwnd);
			ShowWindow(hwnd, SW_RESTORE);
		}
		PostAppMessage(GetCurrentTask(), WM_QUIT, 0, 0);
	} else
		TApplication::InitInstance();
}


int OwlMain(int /*argc*/, char* /*argv*/[])
{
	TOptionApp app("AlwaysOnTop");
	return app.Run();
}
